@@ -43,20 +43,30 @@ class ClipboardHandler:
         self.clipboard = gtk.clipboard_get()
         self.force_plain_text = False
 
-    def copy(self, sourceview):
+    def copy(self, text_view, from_codebox):
         """Copy to Clipboard"""
-        sourceview.stop_emission("copy-clipboard")
-        if not self.dad.curr_buffer.get_has_selection(): return
-        self.selection_to_clipboard(self.dad.curr_buffer, sourceview)
+        text_buffer = text_view.get_buffer()
+        if not text_buffer.get_has_selection(): return
+        iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
+        num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
+        if from_codebox or self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
+            if num_chars > 30000: return
+        text_view.stop_emission("copy-clipboard")
+        self.selection_to_clipboard(text_buffer, text_view, iter_sel_start, iter_sel_end, num_chars, from_codebox)
 
-    def cut(self, sourceview):
+    def cut(self, text_view, from_codebox):
         """Cut to Clipboard"""
-        sourceview.stop_emission("cut-clipboard")
-        if not self.dad.curr_buffer.get_has_selection(): return
-        self.selection_to_clipboard(self.dad.curr_buffer, sourceview)
-        if not self.dad.treestore[self.dad.curr_tree_iter][7]:
-            self.dad.curr_buffer.delete_selection(True, sourceview.get_editable())
-            self.dad.sourceview.grab_focus()
+        text_buffer = text_view.get_buffer()
+        if not text_buffer.get_has_selection(): return
+        iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
+        num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
+        if from_codebox or self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
+            if num_chars > 30000: return
+        text_view.stop_emission("cut-clipboard")
+        self.selection_to_clipboard(text_buffer, text_view, iter_sel_start, iter_sel_end, num_chars, from_codebox)
+        if self.dad.is_curr_node_not_read_only_or_error():
+            text_buffer.delete_selection(True, text_view.get_editable())
+            text_view.grab_focus()
 
     def table_row_to_clipboard(self, table_dict):
         """Put the Selected Table Row to the Clipboard"""
@@ -64,7 +74,7 @@ class ClipboardHandler:
         self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_TABLE, TARGETS_HTML[0])],
                                      self.get_func,
                                      self.clear_func,
-                                     (table_dict, None, html_text))
+                                     (table_dict, None, html_text, None))
 
     def table_row_paste(self, model_n_iter):
         """Paste Table Row from the Clipboard"""
@@ -75,27 +85,22 @@ class ClipboardHandler:
             return True
         return False
 
-    def selection_to_clipboard(self, text_buffer, sourceview):
+    def selection_to_clipboard(self, text_buffer, sourceview, iter_sel_start, iter_sel_end, num_chars, from_codebox):
         """Write the Selected Content to the Clipboard"""
-        iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
-        num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
-        if num_chars == 1:
+        pixbuf_target = None
+        if not from_codebox and self.dad.syntax_highlighting == cons.RICH_TEXT_ID and num_chars == 1:
             anchor = iter_sel_start.get_child_anchor()
             if anchor:
                 anchor_dir = dir(anchor)
                 if "pixbuf" in anchor_dir:
-                    self.clipboard.set_with_data([(t, 0, 0) for t in TARGETS_IMAGES],
-                                                 self.get_func,
-                                                 self.clear_func,
-                                                 anchor.pixbuf)
-                    return
+                    pixbuf_target = anchor.pixbuf
                 elif "liststore" in anchor_dir:
                     table_dict = self.dad.state_machine.table_to_dict(anchor)
                     html_text = self.dad.html_handler.table_export_to_html(table_dict)
                     self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_TABLE, TARGETS_HTML[0])],
                                                  self.get_func,
                                                  self.clear_func,
-                                                 (table_dict, None, html_text))
+                                                 (table_dict, None, html_text, None))
                     return
                 elif "sourcebuffer" in anchor_dir:
                     codebox_dict = self.dad.state_machine.codebox_to_dict(anchor, for_print=0)
@@ -104,24 +109,27 @@ class ClipboardHandler:
                     self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_CODEBOX, TARGETS_HTML[0])],
                                                  self.get_func,
                                                  self.clear_func,
-                                                 (codebox_dict, None, html_text))
+                                                 (codebox_dict, None, html_text, None))
                     return
         if not os.path.isdir(cons.TMP_FOLDER): os.mkdir(cons.TMP_FOLDER)
-        html_text = self.dad.html_handler.selection_export_to_html(text_buffer, iter_sel_start, iter_sel_end, self.dad.syntax_highlighting)
-        if self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
+        html_text = self.dad.html_handler.selection_export_to_html(text_buffer, iter_sel_start, iter_sel_end,
+            self.dad.syntax_highlighting if not from_codebox else cons.PLAIN_TEXT_ID)
+        if not from_codebox and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             txt_handler = exports.Export2Txt(self.dad)
             plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=[iter_sel_start.get_offset(), iter_sel_end.get_offset()])
             rich_text = self.rich_text_get_from_text_buffer_selection(text_buffer, iter_sel_start, iter_sel_end)
-            self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0])],
-                                         self.get_func,
-                                         self.clear_func,
-                                         (plain_text, rich_text, html_text))
+            targets_vector = [TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0]]
+            if pixbuf_target: targets_vector.append(TARGETS_IMAGES[0])
+            self.clipboard.set_with_data([(t, 0, 0) for t in targets_vector],
+                self.get_func,
+                self.clear_func,
+                (plain_text, rich_text, html_text, pixbuf_target))
         else:
             plain_text = text_buffer.get_text(iter_sel_start, iter_sel_end)
             self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_PLAIN_TEXT, TARGETS_HTML[0])],
                                          self.get_func,
                                          self.clear_func,
-                                         (plain_text, None, html_text))
+                                         (plain_text, None, html_text, pixbuf_target))
 
     def get_func(self, clipboard, selectiondata, info, data):
         """Connected with clipboard.set_with_data"""
@@ -137,7 +145,7 @@ class ClipboardHandler:
             dom = xml.dom.minidom.Document()
             self.dad.xml_handler.table_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom, dom)
             selectiondata.set('UTF8_STRING', 8, dom.toxml())
-        elif target in TARGETS_IMAGES: selectiondata.set_pixbuf(data)
+        elif target == TARGETS_IMAGES[0]: selectiondata.set_pixbuf(data[3])
 
     def clear_func(self, clipboard, data):
         """Connected with clipboard.set_with_data"""
@@ -241,7 +249,8 @@ class ClipboardHandler:
         else:
             match = re.match('.*\x00\w\x00\w\x00\w.*', selectiondata.data, re.UNICODE) # \w is alphanumeric char
             if match: selection_data = selectiondata.data.decode(cons.STR_UTF16, cons.STR_IGNORE)
-            elif "HTML" in selectiondata.data: selection_data = selectiondata.data.decode(cons.STR_UTF8, cons.STR_IGNORE)
+            elif "html" in selectiondata.data or "HTML" in selectiondata.data:
+                selection_data = selectiondata.data.decode(cons.STR_UTF8, cons.STR_IGNORE)
             else:
                 utf8_OK = False
                 utf16_OK = False
@@ -256,7 +265,7 @@ class ClipboardHandler:
                 except: pass
                 if utf16_OK == utf8_OK:
                     if cons.IS_WIN_OS: selection_data = selectiondata.data.decode(cons.STR_UTF16, cons.STR_IGNORE)
-                    else: selection_data = selectiondata.data
+                    else: selection_data = selectiondata.data.decode(cons.STR_UTF8, cons.STR_IGNORE)
         #print "###########################"
         #print selectiondata.data
         #print "###########################"
@@ -369,6 +378,7 @@ class ClipboardHandler:
             pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.FILE_CHAR, self.dad.embfile_size, self.dad.embfile_size)
             pixbuf.filename = dom_node.attributes["filename"].value
             pixbuf.embfile = base64.b64decode(dom_node.firstChild.data)
+            pixbuf.time = float(dom_node.attributes["time"].value) if dom_node.hasAttribute("time") else 0
         else: pixbuf = machines.get_pixbuf_from_encoded_buffer(dom_node.firstChild.data)
         if pixbuf:
             pixbuf.link = dom_node.attributes["link"].value if dom_node.hasAttribute("link") else ""
