@@ -24,7 +24,7 @@ import os
 import cons, support
 
 DRAW_SPACES_FLAGS = gtksourceview2.DRAW_SPACES_ALL & ~gtksourceview2.DRAW_SPACES_NEWLINE
-CB_WIDTH_HEIGHT_STEP_PIX = 30
+CB_WIDTH_HEIGHT_STEP_PIX = 15
 CB_WIDTH_HEIGHT_STEP_PERC = 9
 CB_WIDTH_LIMIT_MIN = 40
 CB_HEIGHT_LIMIT_MIN = 30
@@ -36,6 +36,19 @@ class CodeBoxesHandler:
     def __init__(self, dad):
         """Lists Handler boot"""
         self.dad = dad
+        self.curr_codebox_anchor = None
+        self.curr_v = 0
+        self.curr_h = 0
+
+    def codebox_in_use(self):
+        """Returns a CodeBox SourceView if Currently in Use or None"""
+        if not self.curr_codebox_anchor: return None
+        if not self.dad.curr_buffer: return None
+        iter_sel_start = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
+        anchor = iter_sel_start.get_child_anchor()
+        if not anchor: return None
+        if "sourcebuffer" in dir(anchor): return anchor.sourceview
+        return None
 
     def codebox_cut(self, *args):
         """Cut CodeBox"""
@@ -49,6 +62,7 @@ class CodeBoxesHandler:
 
     def codebox_delete_keeping_text(self, *args):
         """Delete CodeBox but keep the Text"""
+        if not self.dad.is_curr_node_not_read_only_or_error(): return
         content = self.curr_codebox_anchor.sourcebuffer.get_text(*self.curr_codebox_anchor.sourcebuffer.get_bounds())
         self.dad.object_set_selection(self.curr_codebox_anchor)
         self.dad.curr_buffer.delete_selection(True, self.dad.sourceview.get_editable())
@@ -61,7 +75,7 @@ class CodeBoxesHandler:
         self.dad.curr_buffer.delete_selection(True, self.dad.sourceview.get_editable())
         self.dad.sourceview.grab_focus()
 
-    def dialog_codeboxhandle(self, title, line_num, match_bra, syntax_highl=cons.PLAIN_TEXT_ID):
+    def dialog_codeboxhandle(self, title):
         """Opens the CodeBox Handle Dialog"""
         dialog = gtk.Dialog(title=title,
                             parent=self.dad.window,
@@ -75,13 +89,13 @@ class CodeBoxesHandler:
         cell = gtk.CellRendererText()
         combobox_prog_lang.pack_start(cell, True)
         combobox_prog_lang.add_attribute(cell, 'text', 0)
-        combobox_value = syntax_highl if syntax_highl != cons.PLAIN_TEXT_ID else self.dad.auto_syn_highl
+        combobox_value = self.dad.codebox_syn_highl if self.dad.codebox_syn_highl != cons.PLAIN_TEXT_ID else self.dad.auto_syn_highl
         combobox_iter = self.dad.get_combobox_iter_from_value(self.dad.prog_lang_liststore, 1, combobox_value)
         combobox_prog_lang.set_active_iter(combobox_iter)
         radiobutton_plain_text = gtk.RadioButton(label=_("Plain Text"))
         radiobutton_auto_syntax_highl = gtk.RadioButton(label=_("Automatic Syntax Highlighting"))
         radiobutton_auto_syntax_highl.set_group(radiobutton_plain_text)
-        if syntax_highl == cons.PLAIN_TEXT_ID:
+        if self.dad.codebox_syn_highl == cons.PLAIN_TEXT_ID:
             radiobutton_plain_text.set_active(True)
             combobox_prog_lang.set_sensitive(False)
         else:
@@ -135,9 +149,9 @@ class CodeBoxesHandler:
         size_frame.add(size_align)
         
         checkbutton_codebox_linenumbers = gtk.CheckButton(label=_("Show Line Numbers"))
-        checkbutton_codebox_linenumbers.set_active(line_num)
+        checkbutton_codebox_linenumbers.set_active(self.dad.codebox_line_num)
         checkbutton_codebox_matchbrackets = gtk.CheckButton(label=_("Highlight Matching Brackets"))
-        checkbutton_codebox_matchbrackets.set_active(match_bra)
+        checkbutton_codebox_matchbrackets.set_active(self.dad.codebox_match_bra)
         vbox_options = gtk.VBox()
         vbox_options.pack_start(checkbutton_codebox_linenumbers)
         vbox_options.pack_start(checkbutton_codebox_matchbrackets)
@@ -182,15 +196,14 @@ class CodeBoxesHandler:
             self.dad.codebox_width = spinbutton_width.get_value()
             self.dad.codebox_width_pixels = radiobutton_codebox_pixels.get_active()
             self.dad.codebox_height = spinbutton_height.get_value()
-            ret_line_num = checkbutton_codebox_linenumbers.get_active()
-            ret_match_bra = checkbutton_codebox_matchbrackets.get_active()
+            self.dad.codebox_line_num = checkbutton_codebox_linenumbers.get_active()
+            self.dad.codebox_match_bra = checkbutton_codebox_matchbrackets.get_active()
             if radiobutton_plain_text.get_active():
-                ret_syntax = cons.PLAIN_TEXT_ID
+                self.dad.codebox_syn_highl = cons.PLAIN_TEXT_ID
             else:
-                ret_syntax = self.dad.prog_lang_liststore[combobox_prog_lang.get_active_iter()][1]
-                self.dad.auto_syn_highl = ret_syntax
-            return [ret_line_num, ret_match_bra, ret_syntax]
-        return [None, None, None]
+                self.dad.codebox_syn_highl = self.dad.prog_lang_liststore[combobox_prog_lang.get_active_iter()][1]
+            return True
+        return False
 
     def codebox_handle(self):
         """Insert Code Box"""
@@ -198,22 +211,21 @@ class CodeBoxesHandler:
             iter_sel_start, iter_sel_end = self.dad.curr_buffer.get_selection_bounds()
             fill_text = unicode(self.dad.curr_buffer.get_text(iter_sel_start, iter_sel_end), cons.STR_UTF8, cons.STR_IGNORE)
         else: fill_text = None
-        ret_line_num, ret_match_bra, ret_syn_highl = self.dialog_codeboxhandle(_("Insert a CodeBox"), False, True)
-        if ret_line_num == None: return
+        if not self.dialog_codeboxhandle(_("Insert a CodeBox")): return
         codebox_dict = {
            'frame_width': int(self.dad.codebox_width),
            'frame_height': int(self.dad.codebox_height),
            'width_in_pixels': self.dad.codebox_width_pixels,
-           'syntax_highlighting': ret_syn_highl,
-           'highlight_brackets': ret_match_bra,
-           'show_line_numbers': ret_line_num,
+           'syntax_highlighting': self.dad.codebox_syn_highl,
+           'highlight_brackets': self.dad.codebox_match_bra,
+           'show_line_numbers': self.dad.codebox_line_num,
            'fill_text': fill_text
         }
         if fill_text: self.dad.curr_buffer.delete(iter_sel_start, iter_sel_end)
         iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
         self.codebox_insert(iter_insert, codebox_dict)
 
-    def codebox_insert(self, iter_insert, codebox_dict, codebox_justification=None, text_buffer=None):
+    def codebox_insert(self, iter_insert, codebox_dict, codebox_justification=None, text_buffer=None, cursor_pos=0):
         """Insert Code Box"""
         if not text_buffer: text_buffer = self.dad.curr_buffer
         anchor = text_buffer.create_child_anchor(iter_insert)
@@ -234,6 +246,7 @@ class CodeBoxesHandler:
         anchor.sourcebuffer.connect('modified-changed', self.dad.on_modified_changed)
         if codebox_dict['fill_text']:
             anchor.sourcebuffer.set_text(codebox_dict['fill_text'])
+            anchor.sourcebuffer.place_cursor(anchor.sourcebuffer.get_iter_at_offset(cursor_pos))
             anchor.sourcebuffer.set_modified(False)
         anchor.sourceview = gtksourceview2.View(anchor.sourcebuffer)
         anchor.sourceview.set_smart_home_end(gtksourceview2.SMART_HOME_END_BEFORE)
@@ -250,19 +263,27 @@ class CodeBoxesHandler:
         anchor.sourceview.connect('populate-popup', self.on_sourceview_populate_popup_codebox, anchor)
         anchor.sourceview.connect('key_press_event', self.on_key_press_sourceview_codebox, anchor)
         anchor.sourceview.connect('button-press-event', self.on_mouse_button_clicked_codebox, anchor)
+        anchor.sourceview.connect("event-after", self.on_sourceview_event_after_codebox, anchor)
+        anchor.sourceview.connect("motion-notify-event", self.on_sourceview_motion_notify_event_codebox)
+        anchor.sourceview.connect("copy-clipboard", self.dad.clipboard_handler.copy, True)
+        anchor.sourceview.connect("cut-clipboard", self.dad.clipboard_handler.cut, True)
         if self.dad.line_wrapping: anchor.sourceview.set_wrap_mode(gtk.WRAP_WORD)
         else: anchor.sourceview.set_wrap_mode(gtk.WRAP_NONE)
-        scrolledwindow = gtk.ScrolledWindow()
-        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        scrolledwindow.add(anchor.sourceview)
+        anchor.scrolledwindow = gtk.ScrolledWindow()
+        anchor.scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        anchor.scrolledwindow.add(anchor.sourceview)
+        anchor.scrolledwindow.get_vscrollbar().connect('event-after', self.on_vscrollbar_event_after, anchor)
+        anchor.scrolledwindow.get_hscrollbar().connect('event-after', self.on_hscrollbar_event_after, anchor)
         anchor.frame = gtk.Frame()
         self.codebox_apply_width_height(anchor)
-        anchor.frame.add(scrolledwindow)
-        self.dad.sourceview.add_child_at_anchor(anchor.frame, anchor)
+        anchor.frame.add(anchor.scrolledwindow)
+        anchor.frame.set_shadow_type(gtk.SHADOW_NONE)
+        anchor.eventbox = gtk.EventBox()
+        anchor.eventbox.add(anchor.frame)
+        self.dad.sourceview.add_child_at_anchor(anchor.eventbox, anchor)
+        anchor.eventbox.show_all()
         for win in [gtk.TEXT_WINDOW_LEFT, gtk.TEXT_WINDOW_RIGHT, gtk.TEXT_WINDOW_TOP, gtk.TEXT_WINDOW_BOTTOM]:
             anchor.sourceview.set_border_window_size(win, 1)
-        anchor.frame.set_shadow_type(gtk.SHADOW_NONE)
-        anchor.frame.show_all()
         if codebox_justification:
             text_iter = text_buffer.get_iter_at_child_anchor(anchor)
             self.dad.state_machine.apply_object_justification(text_iter, codebox_justification, text_buffer)
@@ -272,6 +293,7 @@ class CodeBoxesHandler:
 
     def codebox_increase_width(self, *args):
         """Increase CodeBox Width"""
+        if self.dad.is_curr_node_read_only(): return
         if self.curr_codebox_anchor.width_in_pixels:
             self.codebox_change_width_height(self.curr_codebox_anchor.frame_width + CB_WIDTH_HEIGHT_STEP_PIX, 0)
         else:
@@ -279,6 +301,7 @@ class CodeBoxesHandler:
 
     def codebox_decrease_width(self, *args):
         """Decrease CodeBox Width"""
+        if self.dad.is_curr_node_read_only(): return
         if self.curr_codebox_anchor.width_in_pixels:
             if self.curr_codebox_anchor.frame_width - CB_WIDTH_HEIGHT_STEP_PIX >= CB_WIDTH_LIMIT_MIN:
                 self.codebox_change_width_height(self.curr_codebox_anchor.frame_width - CB_WIDTH_HEIGHT_STEP_PIX, 0)
@@ -288,10 +311,12 @@ class CodeBoxesHandler:
 
     def codebox_increase_height(self, *args):
         """Increase CodeBox Height"""
+        if self.dad.is_curr_node_read_only(): return
         self.codebox_change_width_height(0, self.curr_codebox_anchor.frame_height + CB_WIDTH_HEIGHT_STEP_PIX)
 
     def codebox_decrease_height(self, *args):
         """Decrease CodeBox Height"""
+        if self.dad.is_curr_node_read_only(): return
         if self.curr_codebox_anchor.frame_height - CB_WIDTH_HEIGHT_STEP_PIX >= CB_HEIGHT_LIMIT_MIN:
             self.codebox_change_width_height(0, self.curr_codebox_anchor.frame_height - CB_WIDTH_HEIGHT_STEP_PIX)
 
@@ -305,23 +330,26 @@ class CodeBoxesHandler:
 
     def codebox_change_width_height(self, new_width, new_height):
         """Replace CodeBox changing Width and Height"""
+        if self.dad.user_active:
+            self.dad.user_active = False
+            user_active_restore = True
+        else: user_active_restore = False
         codebox_iter = self.dad.curr_buffer.get_iter_at_child_anchor(self.curr_codebox_anchor)
         codebox_element = [codebox_iter.get_offset(),
-                           self.dad.state_machine.codebox_to_dict(self.curr_codebox_anchor,
-                                                                  for_print=0),
+                           self.dad.state_machine.codebox_to_dict(self.curr_codebox_anchor, for_print=0),
                            self.dad.state_machine.get_iter_alignment(codebox_iter)]
         if new_width: codebox_element[1]['frame_width'] = new_width
         if new_height: codebox_element[1]['frame_height'] = new_height
-        #cursor_pos_restore = self.curr_codebox_anchor.sourcebuffer.get_property(cons.STR_CURSOR_POSITION)
+        cursor_pos = self.curr_codebox_anchor.sourcebuffer.get_property(cons.STR_CURSOR_POSITION)
         self.codebox_delete()
         iter_insert = self.dad.curr_buffer.get_iter_at_offset(codebox_element[0])
-        self.codebox_insert(iter_insert, codebox_element[1], codebox_element[2])
+        self.codebox_insert(iter_insert, codebox_element[1], codebox_element[2], cursor_pos=cursor_pos)
         self.curr_codebox_anchor.sourceview.grab_focus()
-        #self.curr_codebox_anchor.sourcebuffer.place_cursor(self.curr_codebox_anchor.sourcebuffer.get_iter_at_offset(cursor_pos_restore))
-        #self.curr_codebox_anchor.sourceview.scroll_to_mark(self.curr_codebox_anchor.sourcebuffer.get_insert(), 0.3)
+        if user_active_restore: self.dad.user_active = True
 
     def codebox_load_from_file(self, action):
         """Load the CodeBox Content From a Text File"""
+        if not self.dad.is_curr_node_not_read_only_or_error(): return
         filepath = support.dialog_file_select(curr_folder=self.dad.pick_dir, parent=self.dad.window)
         if not filepath: return
         self.dad.pick_dir = os.path.dirname(filepath)
@@ -339,12 +367,15 @@ class CodeBoxesHandler:
 
     def codebox_change_properties(self, action):
         """Change CodeBox Properties"""
+        if not self.dad.is_curr_node_not_read_only_or_error(): return
         self.dad.codebox_width = self.curr_codebox_anchor.frame_width
         self.dad.codebox_width_pixels = self.curr_codebox_anchor.width_in_pixels
         self.dad.codebox_height = self.curr_codebox_anchor.frame_height
-        ret_line_num, ret_match_bra, ret_syn_highl = self.dialog_codeboxhandle(_("Edit CodeBox"), self.curr_codebox_anchor.show_line_numbers, self.curr_codebox_anchor.highlight_brackets, self.curr_codebox_anchor.syntax_highlighting)
-        if ret_line_num == None: return
-        self.curr_codebox_anchor.syntax_highlighting = ret_syn_highl
+        self.dad.codebox_line_num = self.curr_codebox_anchor.show_line_numbers
+        self.dad.codebox_match_bra = self.curr_codebox_anchor.highlight_brackets
+        self.dad.codebox_syn_highl = self.curr_codebox_anchor.syntax_highlighting
+        if not self.dialog_codeboxhandle(_("Edit CodeBox")): return
+        self.curr_codebox_anchor.syntax_highlighting = self.dad.codebox_syn_highl
         self.dad.set_sourcebuffer_syntax_highlight(self.curr_codebox_anchor.sourcebuffer, self.curr_codebox_anchor.syntax_highlighting)
         if self.curr_codebox_anchor.syntax_highlighting == cons.PLAIN_TEXT_ID:
             self.curr_codebox_anchor.sourceview.modify_font(pango.FontDescription(self.dad.text_font))
@@ -353,13 +384,43 @@ class CodeBoxesHandler:
         self.curr_codebox_anchor.frame_width = int(self.dad.codebox_width)
         self.curr_codebox_anchor.frame_height = int(self.dad.codebox_height)
         self.curr_codebox_anchor.width_in_pixels = self.dad.codebox_width_pixels
-        self.curr_codebox_anchor.highlight_brackets = ret_match_bra
+        self.curr_codebox_anchor.highlight_brackets = self.dad.codebox_match_bra
         self.curr_codebox_anchor.sourcebuffer.set_highlight_matching_brackets(self.curr_codebox_anchor.highlight_brackets)
-        self.curr_codebox_anchor.show_line_numbers = ret_line_num
+        self.curr_codebox_anchor.show_line_numbers = self.dad.codebox_line_num
         self.curr_codebox_anchor.sourceview.set_show_line_numbers(self.curr_codebox_anchor.show_line_numbers)
         self.codebox_apply_width_height(self.curr_codebox_anchor)
         self.dad.update_window_save_needed("nbuf", True)
 
+    def on_sourceview_event_after_codebox(self, text_view, event, anchor):
+        """Called after every event on the SourceView"""
+        if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
+            support.on_sourceview_event_after_double_click_button1(self.dad, text_view, event)
+        elif event.type == gtk.gdk.BUTTON_PRESS:
+            return support.on_sourceview_event_after_button_press(self.dad, text_view, event)
+        elif event.type == gtk.gdk.KEY_PRESS:
+            return support.on_sourceview_event_after_key_press(self.dad, text_view, event)
+        return False
+
+    def on_vscrollbar_event_after(self, vscrollbar, event, anchor):
+        """Catches CodeBox Vertical Scrollbar Movements"""
+        if not self.dad.user_active or self.curr_codebox_anchor != anchor: return False
+        if self.dad.codebox_auto_resize and event.type == gtk.gdk.EXPOSE:
+            curr_v = vscrollbar.get_value()
+            if curr_v:
+                self.curr_v = curr_v
+                if not self.dad.codebox_sentinel_id: self.dad.codebox_sentinel_start()
+        return False
+
+    def on_hscrollbar_event_after(self, hscrollbar, event, anchor):
+        """Catches CodeBox Horizontal Scrollbar Movements"""
+        if not self.dad.user_active or self.curr_codebox_anchor != anchor: return False
+        if self.dad.codebox_auto_resize and event.type == gtk.gdk.EXPOSE:
+            curr_h = hscrollbar.get_value()
+            if curr_h:
+                self.curr_h = curr_h
+                if not self.dad.codebox_sentinel_id: self.dad.codebox_sentinel_start()
+        return False
+
     def on_key_press_sourceview_codebox(self, widget, event, anchor):
         """Extend the Default Right-Click Menu of the CodeBox"""
         if event.state & gtk.gdk.CONTROL_MASK:
@@ -373,13 +434,22 @@ class CodeBoxesHandler:
                 if event.state & gtk.gdk.MOD1_MASK:
                     self.codebox_decrease_height()
                 else: self.codebox_increase_height()
+        return False
 
     def on_mouse_button_clicked_codebox(self, widget, event, anchor):
         """Catches mouse buttons clicks"""
         self.curr_codebox_anchor = anchor
         if event.button != 3:
             self.dad.object_set_selection(self.curr_codebox_anchor)
+        return False
 
     def on_sourceview_populate_popup_codebox(self, textview, menu, anchor):
         """Extend the Default Right-Click Menu of the CodeBox"""
         self.dad.menu_populate_popup(menu, cons.get_popup_menu_entries_codebox(self), self.dad.orphan_accel_group)
+
+    def on_sourceview_motion_notify_event_codebox(self, text_view, event):
+        """Update the cursor image if the pointer moved"""
+        text_view.set_editable(not self.dad.is_curr_node_read_only())
+        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
+        support.sourceview_cursor_and_tooltips_handler(self.dad, text_view, x, y)
+        return False
