@@ -34,9 +34,40 @@ COLOR_PALETTE_DEFAULT = ["#000000", "#ffffff", "#7f7f7f", "#ff0000", "#a020f0",
                          "#e6e6fa", "#a52a2a", "#8b6914", "#1e90ff", "#ffc0cb",
                          "#90ee90", "#1a1a1a", "#4d4d4d", "#bfbfbf", "#e5e5e5"]
 SPECIAL_CHARS_DEFAULT = "“”„•☐☑☒…‰€©®™°↓↑→←↔↵⇓⇑⇒⇐⇔»«▼▲►◄≤≥≠±¹²³½¼⅛×÷∞ø∑√∫ΔδΠπΣΦΩωαβγεηλμ☺☻☼♥♀♂♪♫"
+SELWORD_CHARS_DEFAULT = ".-@"
+TOOLBAR_VEC_DEFAULT = ["TreeAddNode", "TreeAddSubNode", cons.TAG_SEPARATOR, "GoBack", "GoForward", cons.TAG_SEPARATOR, cons.CHAR_STAR, "Save", "Export2PDF", cons.TAG_SEPARATOR, "FindInNodes", cons.TAG_SEPARATOR, "BulletedList", "NumberedList", "ToDoList", cons.TAG_SEPARATOR, "HandleImage", "HandleTable", "HandleCodeBox", "EmbFileInsert", "HandleLink", "HandleAnchor", cons.TAG_SEPARATOR, "RemoveFormatting", "ColorForeground", "ColorBackground", "Bold", "Italic", "Underline", "Strikethrough", "H1", "H2", "H3", "Small", "Superscript", "Subscript", "Monospace"]
+TOOLBAR_VEC_BLACKLIST = ["CutAnchor", "CopyAnchor", "DeleteAnchor", "EditAnchor", "CutEmbFile", "CopyEmbFile", "DeleteEmbFile", "EmbFileSave", "EmbFileOpen", "SaveImage", "EditImage", "CutImage", "CopyImage", "DeleteImage", "EditImageLink", "DismissImageLink", "ShowHideMainWin"]
+SEPARATOR_ASCII_REPR = "---------"
 
 SPELL_CHECK_LANG_DEFAULT = locale.getdefaultlocale()[0]
 
+def get_toolbar_entry_columns_from_key(dad, key):
+    if key == cons.TAG_SEPARATOR: return [key, "", SEPARATOR_ASCII_REPR]
+    if key == cons.CHAR_STAR: return [key, "gtk-open", _("Open a CherryTree Document")]
+    for element in cons.get_entries(dad):
+        if len(element) == 3: continue
+        if key == element[0]: return [element[0], element[1], element[4]]
+    return ["", "", ""]
+
+def get_toolbar_icon_n_label_list(dad):
+    icon_n_label_list = [[cons.TAG_SEPARATOR, "", SEPARATOR_ASCII_REPR]]
+    for element in cons.get_entries(dad):
+        if len(element) == 3: continue
+        if element[0] in dad.toolbar_ui_vec: continue
+        if element[0] in TOOLBAR_VEC_BLACKLIST: continue
+        if element[0] == "OpenFile" and cons.CHAR_STAR in dad.toolbar_ui_vec: continue
+        icon_n_label_list.append([element[0], element[1], element[4]])
+    return icon_n_label_list
+
+def get_toolbar_ui_str(dad):
+    dad.toolbar_open_n_recent = -1
+    toolbar_ui_str = "<ui><toolbar name='ToolBar'>"
+    for i, toolbar_element in enumerate(dad.toolbar_ui_vec):
+        if toolbar_element == cons.CHAR_STAR: dad.toolbar_open_n_recent = i
+        elif toolbar_element == cons.TAG_SEPARATOR: toolbar_ui_str += "<separator/>"
+        else: toolbar_ui_str += "<toolitem action='%s'/>" % toolbar_element
+    return toolbar_ui_str + "</toolbar></ui>"
+
 def get_node_path_from_str(str_path_list_of_str):
     str_path_list_of_str = str(str_path_list_of_str)
     if str_path_list_of_str.startswith(cons.CHAR_PARENTH_OPEN)\
@@ -131,7 +162,9 @@ def config_file_load(inst):
         inst.highl_curr_line = config.getboolean(section, "highl_curr_line") if config.has_option(section, "highl_curr_line") else True
         inst.h_rule = config.get(section, "h_rule") if config.has_option(section, "h_rule") else HORIZONTAL_RULE
         inst.special_chars = unicode(config.get(section, "special_chars") if config.has_option(section, "special_chars") else SPECIAL_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
+        inst.selword_chars = unicode(config.get(section, "selword_chars") if config.has_option(section, "selword_chars") else SELWORD_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
         inst.timestamp_format = config.get(section, "timestamp_format") if config.has_option(section, "timestamp_format") else "%Y/%m/%d - %H:%M"
+        inst.links_relative = config.getboolean(section, "links_relative") if config.has_option(section, "links_relative") else False
         if config.has_option(section, "weblink_custom_action"):
             temp_str = config.get(section, "weblink_custom_action")
             inst.weblink_custom_action = [True, temp_str[4:]] if temp_str[:4] == "True" else [False, temp_str[5:]]
@@ -153,6 +186,10 @@ def config_file_load(inst):
             inst.codebox_height = config.getfloat(section, "codebox_height")
         else: inst.codebox_height = 100
         inst.codebox_width_pixels = config.getboolean(section, "codebox_width_pixels") if config.has_option(section, "codebox_width_pixels") else True
+        inst.codebox_line_num = config.getboolean(section, "codebox_line_num") if config.has_option(section, "codebox_line_num") else False
+        inst.codebox_match_bra = config.getboolean(section, "codebox_match_bra") if config.has_option(section, "codebox_match_bra") else True
+        inst.codebox_syn_highl = config.get(section, "codebox_syn_highl") if config.has_option(section, "codebox_syn_highl") else cons.PLAIN_TEXT_ID
+        inst.codebox_auto_resize = config.getboolean(section, "codebox_auto_resize") if config.has_option(section, "codebox_auto_resize") else True
 
         section = "table"
         inst.table_rows = config.getint(section, "table_rows") if config.has_option(section, "table_rows") else 3
@@ -180,6 +217,7 @@ def config_file_load(inst):
         inst.col_link_fold = config.get(section, "col_link_fold") if config.has_option(section, "col_link_fold") else cons.COLOR_48_LINK_FOLD
 
         section = "misc"
+        inst.toolbar_ui_vec = config.get(section, "toolbar_ui_vec").split(cons.CHAR_COMMA) if config.has_option(section, "toolbar_ui_vec") else TOOLBAR_VEC_DEFAULT
         inst.systray = config.getboolean(section, "systray") if config.has_option(section, "systray") else False
         inst.start_on_systray = config.getboolean(section, "start_on_systray") if config.has_option(section, "start_on_systray") else False
         inst.use_appind = config.getboolean(section, "use_appind") if config.has_option(section, "use_appind") else False
@@ -216,6 +254,7 @@ def config_file_load(inst):
         inst.col_link_fold = cons.COLOR_48_LINK_FOLD
         inst.h_rule = HORIZONTAL_RULE
         inst.special_chars = unicode(SPECIAL_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
+        inst.selword_chars = unicode(SELWORD_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
         inst.enable_spell_check = False
         inst.spell_check_lang = SPELL_CHECK_LANG_DEFAULT
         inst.show_line_numbers = False
@@ -226,6 +265,7 @@ def config_file_load(inst):
         inst.line_wrapping = True
         inst.wrapping_indent = -14
         inst.auto_indent = True
+        inst.toolbar_ui_vec = TOOLBAR_VEC_DEFAULT
         inst.systray = False
         inst.win_position = [10, 10]
         inst.autosave = [False, 5]
@@ -247,13 +287,18 @@ def config_file_load(inst):
         inst.cherry_wrap_width = 130
         inst.start_on_systray = False
         inst.use_appind = False
+        inst.links_relative = False
         inst.weblink_custom_action = [False, LINK_CUSTOM_ACTION_DEFAULT_WEB]
         inst.filelink_custom_action = [False, LINK_CUSTOM_ACTION_DEFAULT_FILE]
         inst.folderlink_custom_action = [False, LINK_CUSTOM_ACTION_DEFAULT_FILE]
         inst.timestamp_format = "%Y/%m/%d - %H:%M"
         inst.codebox_width = 700
-        inst.codebox_width_pixels = True
         inst.codebox_height = 100
+        inst.codebox_width_pixels = True
+        inst.codebox_line_num = False
+        inst.codebox_match_bra = True
+        inst.codebox_syn_highl = cons.PLAIN_TEXT_ID
+        inst.codebox_auto_resize = True
         inst.check_version = False
         inst.reload_doc_last = True
         inst.enable_mod_time_sentinel = True
@@ -376,7 +421,9 @@ def config_file_save(inst):
     config.set(section, "highl_curr_line", inst.highl_curr_line)
     config.set(section, "h_rule", inst.h_rule)
     config.set(section, "special_chars", inst.special_chars)
+    config.set(section, "selword_chars", inst.selword_chars)
     config.set(section, "timestamp_format", inst.timestamp_format)
+    config.set(section, "links_relative", inst.links_relative)
     config.set(section, "weblink_custom_action", str(inst.weblink_custom_action[0])+inst.weblink_custom_action[1])
     config.set(section, "filelink_custom_action", str(inst.filelink_custom_action[0])+inst.filelink_custom_action[1])
     config.set(section, "folderlink_custom_action", str(inst.folderlink_custom_action[0])+inst.folderlink_custom_action[1])
@@ -386,6 +433,10 @@ def config_file_save(inst):
     config.set(section, "codebox_width", inst.codebox_width)
     config.set(section, "codebox_height", inst.codebox_height)
     config.set(section, "codebox_width_pixels", inst.codebox_width_pixels)
+    config.set(section, "codebox_line_num", inst.codebox_line_num)
+    config.set(section, "codebox_match_bra", inst.codebox_match_bra)
+    config.set(section, "codebox_syn_highl", inst.codebox_syn_highl)
+    config.set(section, "codebox_auto_resize", inst.codebox_auto_resize)
 
     section = "table"
     config.add_section(section)
@@ -415,6 +466,7 @@ def config_file_save(inst):
 
     section = "misc"
     config.add_section(section)
+    config.set(section, "toolbar_ui_vec", cons.CHAR_COMMA.join(inst.toolbar_ui_vec))
     config.set(section, "systray", inst.systray)
     config.set(section, "start_on_systray", inst.start_on_systray)
     config.set(section, "use_appind", inst.use_appind)
@@ -452,29 +504,31 @@ def get_tree_expanded_collapsed_string_iter(tree_iter, inst):
         tree_iter = inst.treestore.iter_next(tree_iter)
     return expanded_collapsed_string
 
-def set_tree_expanded_collapsed_string(inst):
+def set_tree_expanded_collapsed_string(inst, treeview=None):
     """Parses the String Containing the Info about Expanded and Collapsed Nodes"""
-    inst.treeview.collapse_all()
+    if not treeview: treeview = inst.treeview
+    treestore = treeview.get_model()
+    treeview.collapse_all()
     if not inst.expanded_collapsed_string: return
     expanded_collapsed_dict = {}
     expanded_collapsed_vector = inst.expanded_collapsed_string.split('_')
     for element in expanded_collapsed_vector:
         couple = element.split(',')
         expanded_collapsed_dict[couple[0]] = couple[1]
-    tree_iter = inst.treestore.get_iter_first()
+    tree_iter = treestore.get_iter_first()
     while tree_iter != None:
-        set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, inst)
-        tree_iter = inst.treestore.iter_next(tree_iter)
+        set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, treeview, treestore)
+        tree_iter = treestore.iter_next(tree_iter)
 
-def set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, inst):
+def set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, treeview, treestore):
     """Iter of the Expanded and Collapsed Nodes Parsing"""
-    node_id = str(inst.treestore[tree_iter][3])
+    node_id = str(treestore[tree_iter][3])
     if node_id in expanded_collapsed_dict and expanded_collapsed_dict[node_id] == "True":
-        inst.treeview.expand_row(inst.treestore.get_path(tree_iter), open_all=False)
-    tree_iter = inst.treestore.iter_children(tree_iter)
+        treeview.expand_row(treestore.get_path(tree_iter), open_all=False)
+    tree_iter = treestore.iter_children(tree_iter)
     while tree_iter != None:
-        set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, inst)
-        tree_iter = inst.treestore.iter_next(tree_iter)
+        set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, treeview, treestore)
+        tree_iter = treestore.iter_next(tree_iter)
 
 def set_tree_path_and_cursor_pos(inst):
     """Try to set node path and cursor pos"""
@@ -570,12 +624,20 @@ def preferences_tab_all_nodes(dad, vbox_all_nodes, pref_dialog):
     scrolledwindow_special_chars.add(textview_special_chars)
     hbox_special_chars.pack_start(label_special_chars, expand=False)
     hbox_special_chars.pack_start(frame_special_chars)
+    hbox_selword_chars = gtk.HBox()
+    hbox_selword_chars.set_spacing(4)
+    label_selword_chars = gtk.Label(_("Chars to Select at Double Click"))
+    entry_selword_chars = gtk.Entry()
+    entry_selword_chars.set_text(dad.selword_chars)
+    hbox_selword_chars.pack_start(label_selword_chars, expand=False)
+    hbox_selword_chars.pack_start(entry_selword_chars)
 
     vbox_misc_all = gtk.VBox()
     vbox_misc_all.set_spacing(2)
     vbox_misc_all.pack_start(hbox_timestamp)
     vbox_misc_all.pack_start(hbox_horizontal_rule)
     vbox_misc_all.pack_start(hbox_special_chars)
+    vbox_misc_all.pack_start(hbox_selword_chars)
     frame_misc_all = gtk.Frame(label="<b>"+_("Miscellaneous")+"</b>")
     frame_misc_all.get_label_widget().set_use_markup(True)
     frame_misc_all.set_shadow_type(gtk.SHADOW_NONE)
@@ -629,6 +691,9 @@ def preferences_tab_all_nodes(dad, vbox_all_nodes, pref_dialog):
     def on_entry_horizontal_rule_changed(entry):
         dad.h_rule = entry.get_text()
     entry_horizontal_rule.connect('changed', on_entry_horizontal_rule_changed)
+    def on_entry_selword_chars_changed(entry):
+        dad.selword_chars = entry.get_text()
+    entry_selword_chars.connect('changed', on_entry_selword_chars_changed)
 
 def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
     """Preferences Dialog, Text Nodes Tab"""
@@ -702,6 +767,8 @@ def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
     hbox_misc_text.set_spacing(4)
     checkbutton_rt_show_white_spaces = gtk.CheckButton(_("Show White Spaces"))
     checkbutton_rt_show_white_spaces.set_active(dad.rt_show_white_spaces)
+    checkbutton_codebox_auto_resize = gtk.CheckButton(_("Expand CodeBoxes Automatically"))
+    checkbutton_codebox_auto_resize.set_active(dad.codebox_auto_resize)
     hbox_embfile_size = gtk.HBox()
     hbox_embfile_size.set_spacing(4)
     label_embfile_size = gtk.Label(_("Embedded File Icon Size"))
@@ -719,6 +786,7 @@ def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
 
     vbox_misc_text = gtk.VBox()
     vbox_misc_text.pack_start(checkbutton_rt_show_white_spaces, expand=False)
+    vbox_misc_text.pack_start(checkbutton_codebox_auto_resize, expand=False)
     vbox_misc_text.pack_start(hbox_embfile_size, expand=False)
     vbox_misc_text.pack_start(hbox_misc_text, expand=False)
     frame_misc_text = gtk.Frame(label="<b>"+_("Miscellaneous")+"</b>")
@@ -783,6 +851,9 @@ def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
         if dad.syntax_highlighting == cons.RICH_TEXT_ID:
             dad.sourceview.set_draw_spaces(codeboxes.DRAW_SPACES_FLAGS if dad.rt_show_white_spaces else 0)
     checkbutton_rt_show_white_spaces.connect('toggled', on_checkbutton_rt_show_white_spaces_toggled)
+    def on_checkbutton_codebox_auto_resize_toggled(checkbutton):
+        dad.codebox_auto_resize = checkbutton.get_active()
+    checkbutton_codebox_auto_resize.connect('toggled', on_checkbutton_codebox_auto_resize_toggled)
     def on_spinbutton_embfile_size_value_changed(spinbutton):
         dad.embfile_size = int(spinbutton_embfile_size.get_value())
         if not dad.embfile_size_mod:
@@ -1085,8 +1156,11 @@ def preferences_tab_fonts(dad, vbox_fonts, pref_dialog):
     fontbutton_text.connect('font-set', on_fontbutton_text_font_set)
     def on_fontbutton_code_font_set(picker):
         dad.code_font = picker.get_font_name()
-        if dad.curr_tree_iter and dad.syntax_highlighting != cons.RICH_TEXT_ID:
+        if not dad.curr_tree_iter: return
+        if dad.syntax_highlighting != cons.RICH_TEXT_ID:
             dad.sourceview.modify_font(pango.FontDescription(dad.code_font))
+        else:
+            support.rich_text_node_modify_codeboxes_font(dad.curr_buffer.get_start_iter(), dad.code_font)
     fontbutton_code.connect('font-set', on_fontbutton_code_font_set)
     def on_fontbutton_tree_font_set(picker):
         dad.tree_font = picker.get_font_name()
@@ -1176,6 +1250,8 @@ def preferences_tab_links(dad, vbox_links, pref_dialog):
     frame_links_colors.add(align_links_colors)
 
     vbox_links_misc = gtk.VBox()
+    checkbutton_links_relative = gtk.CheckButton(_("Use Relative Paths for Files And Folders"))
+    checkbutton_links_relative.set_active(dad.links_relative)
     hbox_anchor_size = gtk.HBox()
     hbox_anchor_size.set_spacing(4)
     label_anchor_size = gtk.Label(_("Anchor Size"))
@@ -1184,6 +1260,7 @@ def preferences_tab_links(dad, vbox_links, pref_dialog):
     spinbutton_anchor_size.set_value(dad.anchor_size)
     hbox_anchor_size.pack_start(label_anchor_size, expand=False)
     hbox_anchor_size.pack_start(spinbutton_anchor_size, expand=False)
+    vbox_links_misc.pack_start(checkbutton_links_relative, expand=False)
     vbox_links_misc.pack_start(hbox_anchor_size, expand=False)
 
     frame_links_misc = gtk.Frame(label="<b>"+_("Miscellaneous")+"</b>")
@@ -1218,6 +1295,9 @@ def preferences_tab_links(dad, vbox_links, pref_dialog):
     def on_entry_custom_folderlink_cmd_changed(entry):
         dad.folderlink_custom_action[1] = entry.get_text()
     entry_custom_folderlink_cmd.connect('changed', on_entry_custom_folderlink_cmd_changed)
+    def on_checkbutton_links_relative_toggled(checkbutton):
+        dad.links_relative = checkbutton.get_active()
+    checkbutton_links_relative.connect('toggled', on_checkbutton_links_relative_toggled)
     def on_spinbutton_anchor_size_value_changed(spinbutton):
         dad.anchor_size = int(spinbutton_anchor_size.get_value())
         if not dad.anchor_size_mod:
@@ -1241,6 +1321,78 @@ def preferences_tab_links(dad, vbox_links, pref_dialog):
         support.dialog_info_after_restart(pref_dialog)
     colorbutton_col_link_fold.connect('color-set', on_colorbutton_col_link_fold_color_set)
 
+def preferences_tab_toolbar(dad, vbox_tool, pref_dialog):
+    """Preferences Dialog, Toolbar Tab"""
+    for child in vbox_tool.get_children(): child.destroy()
+
+    liststore = gtk.ListStore(str, str, str)
+    treeview = gtk.TreeView(liststore)
+    treeview.set_headers_visible(False)
+    treeview.set_reorderable(True)
+    treeview.set_size_request(300, 300)
+    renderer_pixbuf = gtk.CellRendererPixbuf()
+    renderer_text = gtk.CellRendererText()
+    column = gtk.TreeViewColumn()
+    column.pack_start(renderer_pixbuf, False)
+    column.pack_start(renderer_text, True)
+    column.set_attributes(renderer_pixbuf, stock_id=1)
+    column.set_attributes(renderer_text, text=2)
+    treeview.append_column(column)
+    treeviewselection = treeview.get_selection()
+    scrolledwindow = gtk.ScrolledWindow()
+    scrolledwindow.add(treeview)
+
+    button_add = gtk.Button()
+    button_add.set_image(gtk.image_new_from_stock("gtk-add", gtk.ICON_SIZE_BUTTON))
+    button_remove = gtk.Button()
+    button_remove.set_image(gtk.image_new_from_stock("gtk-remove", gtk.ICON_SIZE_BUTTON))
+
+    hbox = gtk.HBox()
+    vbox = gtk.VBox()
+    vbox.pack_start(button_add, expand=False)
+    vbox.pack_start(button_remove, expand=False)
+    hbox.pack_start(scrolledwindow, expand=True)
+    hbox.pack_start(vbox, expand=False)
+
+    vbox_tool.add(hbox)
+
+    for element in dad.toolbar_ui_vec:
+        liststore.append(get_toolbar_entry_columns_from_key(dad, element))
+
+    pref_dialog.disp_dialog_after_restart = False
+    def update_toolbar_ui_vec():
+        dad.toolbar_ui_vec = []
+        tree_iter = liststore.get_iter_first()
+        while tree_iter != None:
+            dad.toolbar_ui_vec.append(liststore[tree_iter][0])
+            tree_iter = liststore.iter_next(tree_iter)
+        if not pref_dialog.disp_dialog_after_restart:
+            pref_dialog.disp_dialog_after_restart = True
+            support.dialog_info_after_restart(pref_dialog)
+    def on_button_add_clicked(*args):
+        icon_n_label_list = get_toolbar_icon_n_label_list(dad)
+        sel_key = support.dialog_choose_element_in_list(pref_dialog, _("Select Element to Add"), [], "", icon_n_label_list)
+        if sel_key:
+            if sel_key == "OpenFile": sel_key = cons.CHAR_STAR
+            model, tree_iter = treeviewselection.get_selected()
+            if tree_iter: liststore.insert_after(tree_iter, get_toolbar_entry_columns_from_key(dad, sel_key))
+            else: liststore.append(get_toolbar_entry_columns_from_key(dad, sel_key))
+            update_toolbar_ui_vec()
+    button_add.connect('clicked', on_button_add_clicked)
+    def on_button_remove_clicked(*args):
+        model, tree_iter = treeviewselection.get_selected()
+        if tree_iter:
+            model.remove(tree_iter)
+            update_toolbar_ui_vec()
+    button_remove.connect('clicked', on_button_remove_clicked)
+    def on_key_press_liststore(widget, event):
+        keyname = gtk.gdk.keyval_name(event.keyval)
+        if keyname == "Delete": on_button_remove_clicked()
+    treeview.connect('key_press_event', on_key_press_liststore)
+    def on_treeview_drag_end(*args):
+        update_toolbar_ui_vec()
+    treeview.connect('drag-end', on_treeview_drag_end)
+
 def preferences_tab_misc(dad, vbox_misc, pref_dialog):
     """Preferences Dialog, Misc Tab"""
     for child in vbox_misc.get_children(): child.destroy()
@@ -1444,7 +1596,7 @@ def dialog_preferences(dad):
         buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT))
 
     tabs_vbox_vec = []
-    for tabs_idx in range(7):
+    for tabs_idx in range(8):
         tabs_vbox_vec.append(gtk.VBox())
         tabs_vbox_vec[-1].set_spacing(3)
 
@@ -1456,7 +1608,8 @@ def dialog_preferences(dad):
     notebook.append_page(tabs_vbox_vec[3], gtk.Label(_("Tree")))
     notebook.append_page(tabs_vbox_vec[4], gtk.Label(_("Fonts")))
     notebook.append_page(tabs_vbox_vec[5], gtk.Label(_("Links")))
-    notebook.append_page(tabs_vbox_vec[6], gtk.Label(_("Miscellaneous")))
+    notebook.append_page(tabs_vbox_vec[6], gtk.Label(_("Toolbar")))
+    notebook.append_page(tabs_vbox_vec[7], gtk.Label(_("Miscellaneous")))
 
     tab_constructor = {
         0: preferences_tab_all_nodes,
@@ -1465,7 +1618,8 @@ def dialog_preferences(dad):
         3: preferences_tab_tree,
         4: preferences_tab_fonts,
         5: preferences_tab_links,
-        6: preferences_tab_misc,
+        6: preferences_tab_toolbar,
+        7: preferences_tab_misc,
         }
 
     def on_notebook_switch_page(notebook, page, page_num):
