@@ -227,6 +227,7 @@ class XMLHandler:
             pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.FILE_CHAR, self.dad.embfile_size, self.dad.embfile_size)
             pixbuf.filename = dom_node.attributes["filename"].value
             pixbuf.embfile = base64.b64decode(dom_node.firstChild.data)
+            pixbuf.time = float(dom_node.attributes["time"].value) if dom_node.hasAttribute("time") else 0
         else:
             if version == 2: pixbuf = get_pixbuf_from_encoded_buffer(dom_node.firstChild.data)
             else: pixbuf = get_pixbuf_from_png_encoded_string(dom_node.firstChild.data)
@@ -401,6 +402,8 @@ class XMLHandler:
             dom_iter.setAttribute("filename", element[1].filename)
             dom_node.appendChild(dom_iter)
             text_iter = dom.createTextNode(base64.b64encode(element[1].embfile))
+            if "time" in dir(element[1]) and element[1].time:
+                dom_iter.setAttribute("time", str(element[1].time))
         else:
             if "link" in dir(element[1]) and element[1].link:
                 dom_iter.setAttribute("link", element[1].link)
@@ -822,6 +825,9 @@ class StateMachine:
 
     def text_variation(self, node_id, varied_text):
         """Insertion or Removal of text in the given node_id"""
+        if cons.CHAR_NEWLINE in varied_text:
+            self.update_state(node_id)
+            return
         alphanum = re.search("\w", varied_text, re.UNICODE) # we search for an alphanumeric character
         if self.nodes_indicators[node_id] < 2:
             if alphanum != None: self.nodes_indicators[node_id] = 2 # alphanumeric transition
@@ -831,15 +837,11 @@ class StateMachine:
 
     def requested_previous_state(self, node_id):
         """A Previous State, if Existing, is Requested"""
-        if self.nodes_indicators[node_id] == 0:
-            if self.nodes_indexes[node_id] == 0: return None
-            else:
-                self.nodes_indexes[node_id] -= 1
-                return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
-        else:
+        if self.curr_index_is_last_index(node_id):
             self.update_state(node_id)
+        if self.nodes_indexes[node_id] > 0:
             self.nodes_indexes[node_id] -= 1
-            return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
+        return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
 
     def requested_current_state(self, node_id):
         """The current state is requested"""
@@ -847,12 +849,9 @@ class StateMachine:
 
     def requested_subsequent_state(self, node_id):
         """A Subsequent State, if Existing, is Requested"""
-        if self.nodes_indicators[node_id] == 0:
-            if self.nodes_indexes[node_id] == len(self.nodes_vectors[node_id]) - 1: return None
-            else:
-                self.nodes_indexes[node_id] += 1
-                return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
-        else: return None
+        if self.nodes_indexes[node_id] < (len(self.nodes_vectors[node_id])-1):
+            self.nodes_indexes[node_id] += 1
+        return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
 
     def delete_states(self, node_id):
         """Delete the states for the given node_id"""
@@ -863,16 +862,25 @@ class StateMachine:
             self.visited_nodes_list.remove(node_id)
             self.visited_nodes_idx = len(self.visited_nodes_list)-1
 
-    def update_state(self, node_id):
-        """Update the state for the given node_id"""
+    def curr_index_is_last_index(self, node_id):
+        """Are we in the last state?"""
         curr_index = self.nodes_indexes[node_id]
         last_index = len(self.nodes_vectors[node_id]) - 1
-        if curr_index != last_index:
-            del self.nodes_vectors[node_id][curr_index+1:]
+        return curr_index == last_index
+
+    def update_state(self, node_id):
+        """Update the state for the given node_id"""
+        if not self.curr_index_is_last_index(node_id):
+            del self.nodes_vectors[node_id][self.nodes_indexes[node_id]+1:]
         xml_content = self.dad.xml_handler.treestore_node_to_dom(self.dad.curr_tree_iter)
         pixbuf_table_codebox_vector = self.get_embedded_pixbufs_tables_codeboxes(self.dad.curr_buffer)
         cursor_pos = self.dad.curr_buffer.get_property(cons.STR_CURSOR_POSITION)
-        self.nodes_vectors[node_id].append([xml_content, pixbuf_table_codebox_vector, cursor_pos])
+        new_state = [xml_content, pixbuf_table_codebox_vector, cursor_pos]
+        if len(self.nodes_vectors[node_id]) > 0:
+            if new_state == self.nodes_vectors[node_id][-1]:
+                #print "update_state not needed"
+                return
+        self.nodes_vectors[node_id].append(new_state)
         num_saved_states = len(self.nodes_vectors[node_id])
         while num_saved_states > self.dad.limit_undoable_steps:
             self.nodes_vectors[node_id].pop(0)
