@@ -19,10 +19,210 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, os
-import cons
+import gtk, pango, os, webbrowser, re
+import cons, config
 
 
+def apply_tag_try_automatic_bounds(dad, text_buffer=None, iter_start=None):
+    """Try to Select a Word Forward/Backward the Cursor"""
+    if not text_buffer: text_buffer = dad.curr_buffer
+    if not iter_start: iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+    iter_end = iter_start.copy()
+    curr_char = iter_end.get_char()
+    # 1) select alphanumeric + special
+    match = re.match('\w', curr_char, re.UNICODE)
+    if not match and not curr_char in dad.selword_chars: return False
+    while match or curr_char in dad.selword_chars:
+        if not iter_end.forward_char(): break # end of buffer
+        curr_char = iter_end.get_char()
+        match = re.match('\w', curr_char, re.UNICODE)
+    iter_start.backward_char()
+    curr_char = iter_start.get_char()
+    match = re.match('\w', curr_char, re.UNICODE)
+    while match or curr_char in dad.selword_chars:
+        if not iter_start.backward_char(): break # start of buffer
+        curr_char = iter_start.get_char()
+        match = re.match('\w', curr_char, re.UNICODE)
+    if not match and not curr_char in dad.selword_chars: iter_start.forward_char()
+    # 2) remove non alphanumeric from borders
+    iter_end.backward_char()
+    curr_char = iter_end.get_char()
+    while curr_char in dad.selword_chars:
+        if not iter_end.backward_char(): break # start of buffer
+        curr_char = iter_end.get_char()
+    iter_end.forward_char()
+    curr_char = iter_start.get_char()
+    while curr_char in dad.selword_chars:
+        if not iter_start.forward_char(): break # end of buffer
+        curr_char = iter_start.get_char()
+    if iter_end.compare(iter_start) > 0:
+        text_buffer.move_mark(text_buffer.get_insert(), iter_start)
+        text_buffer.move_mark(text_buffer.get_selection_bound(), iter_end)
+        return True
+    return False
+
+def on_sourceview_event_after_double_click_button1(dad, text_view, event):
+    """Called after every Double Click with button 1"""
+    text_buffer = text_view.get_buffer()
+    x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
+    iter_start = text_view.get_iter_at_location(x, y)
+    apply_tag_try_automatic_bounds(dad, text_buffer=text_buffer, iter_start=iter_start)
+
+def on_sourceview_event_after_button_press(dad, text_view, event):
+    """Called after every gtk.gdk.BUTTON_PRESS on the SourceView"""
+    text_buffer = text_view.get_buffer()
+    if event.button == 1:
+        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
+        text_iter = text_view.get_iter_at_location(x, y)
+        tags = text_iter.get_tags()
+        # check whether we are hovering a link
+        if not tags: tags = []
+        for tag in tags:
+            tag_name = tag.get_property("name")
+            if tag_name and tag_name[0:4] == cons.TAG_LINK:
+                dad.link_clicked(tag_name[5:])
+                return False
+        if dad.lists_handler.is_list_todo_beginning(text_iter):
+            dad.lists_handler.todo_list_rotate_status(text_iter, text_buffer)
+    elif event.button == 3 and not text_buffer.get_has_selection():
+        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
+        text_iter = text_view.get_iter_at_location(x, y)
+        text_buffer.place_cursor(text_iter)
+    return False
+
+def on_sourceview_event_after_key_press(dad, text_view, event):
+    """Called after every gtk.gdk.KEY_PRESS on the SourceView"""
+    text_buffer = text_view.get_buffer()
+    keyname = gtk.gdk.keyval_name(event.keyval)
+    if (event.state & gtk.gdk.SHIFT_MASK):
+        if keyname == cons.STR_RETURN:
+            text_buffer.insert(text_buffer.get_iter_at_mark(text_buffer.get_insert()), 3*cons.CHAR_SPACE)
+    elif keyname == cons.STR_RETURN:
+        iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+        if not iter_insert:
+            return False
+        cursor_key_press = iter_insert.get_offset()
+        #print "cursor_key_press", cursor_key_press
+        if cursor_key_press == dad.cursor_key_press:
+            # problem of event-after called twice, once before really executing
+            return False
+        iter_start = iter_insert.copy()
+        if not iter_start.backward_char(): return False
+        if iter_start.get_char() != cons.CHAR_NEWLINE: return False
+        if iter_start.backward_char() and iter_start.get_char() == cons.CHAR_NEWLINE:
+            return False # former was an empty row
+        list_info = dad.lists_handler.get_paragraph_list_info(iter_start)
+        if list_info[0] == None:
+            if dad.auto_indent:
+                iter_start = iter_insert.copy()
+                former_line_indent = get_former_line_indentation(iter_start)
+                if former_line_indent: text_buffer.insert_at_cursor(former_line_indent)
+            return False # former was not a list
+        # possible list quit
+        iter_list_quit = iter_insert.copy()
+        if (list_info[0] == 0 and iter_list_quit.backward_chars(3) and iter_list_quit.get_char() == cons.CHAR_LISTBUL):
+            text_buffer.delete(iter_list_quit, iter_insert)
+            return False # former was an empty paragraph => list quit
+        elif (list_info[0] == -1 and iter_list_quit.backward_chars(3) and iter_list_quit.get_char() in [cons.CHAR_LISTTODO, cons.CHAR_LISTDONEOK, cons.CHAR_LISTDONEFAIL]):
+            text_buffer.delete(iter_list_quit, iter_insert)
+            return False # former was an empty paragraph => list quit
+        elif (list_info[0] > 0 and iter_list_quit.backward_chars(2) and iter_list_quit.get_char() == cons.CHAR_SPACE\
+        and iter_list_quit.backward_char() and iter_list_quit.get_char() == '.'):
+            iter_list_quit.backward_chars(len(str(list_info[0])))
+            text_buffer.delete(iter_list_quit, iter_insert)
+            return False # former was an empty paragraph => list quit
+        if list_info[0] == 0: text_buffer.insert(iter_insert, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+        elif list_info[0] == -1: text_buffer.insert(iter_insert, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
+        else:
+            curr_num = list_info[0] + 1
+            text_buffer.insert(iter_insert, '%s. ' % curr_num)
+            dad.lists_handler.list_adjust_ahead(curr_num, iter_insert.get_offset(), "num2num", text_buffer)
+    elif keyname == "space":
+        iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+        if iter_insert == None: return False
+        iter_start = iter_insert.copy()
+        if iter_start.backward_chars(2):
+            if iter_start.get_char() == cons.CHAR_GREATER and iter_start.backward_char()\
+            and iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char():
+                if iter_start.get_char() == cons.CHAR_LESSER:
+                    dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_DOUBLE, iter_start, iter_insert, text_buffer)
+                elif iter_start.get_char() == cons.CHAR_MINUS:
+                    dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_RIGHT, iter_start, iter_insert, text_buffer)
+            elif iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char()\
+            and iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char()\
+            and iter_start.get_char() == cons.CHAR_LESSER:
+                dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_LEFT, iter_start, iter_insert, text_buffer)
+            elif iter_start.get_char() == cons.CHAR_PARENTH_CLOSE and iter_start.backward_char():
+                if iter_start.get_char().lower() == "c" and iter_start.backward_char()\
+                and iter_start.get_char() == cons.CHAR_PARENTH_OPEN:
+                    dad.special_char_replace(cons.SPECIAL_CHAR_COPYRIGHT, iter_start, iter_insert, text_buffer)
+                elif iter_start.get_char().lower() == "r" and iter_start.backward_char()\
+                and iter_start.get_char() == cons.CHAR_PARENTH_OPEN:
+                    dad.special_char_replace(cons.SPECIAL_CHAR_REGISTERED_TRADEMARK, iter_start, iter_insert, text_buffer)
+                elif iter_start.get_char().lower() == "m" and iter_start.backward_char()\
+                and iter_start.get_char() == "t" and iter_start.backward_char()\
+                and iter_start.get_char() == cons.CHAR_PARENTH_OPEN:
+                    dad.special_char_replace(cons.SPECIAL_CHAR_UNREGISTERED_TRADEMARK, iter_start, iter_insert, text_buffer)
+            # Start bulleted list on "* " at line start
+            elif iter_start.get_char() == cons.CHAR_STAR and iter_start.get_line_offset() == 0:
+                text_buffer.delete(iter_start, iter_insert)
+                dad.lists_handler.list_bulleted_handler(text_buffer=text_buffer)
+            # Start todo list on "[ ]" at line start
+            elif iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE and iter_start.backward_char()\
+            and iter_start.get_char() == cons.CHAR_SPACE and iter_start.backward_char()\
+            and iter_start.get_char() == cons.CHAR_SQ_BR_OPEN\
+            and iter_start.get_line_offset() == 0:
+                text_buffer.delete(iter_start, iter_insert)
+                dad.lists_handler.list_todo_handler(text_buffer=text_buffer)
+    return False
+
+def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
+    """Looks at all tags covering the position (x, y) in the text view,
+       and if one of them is a link, change the cursor to the HAND2 cursor"""
+    hovering_link = False
+    text_iter = text_view.get_iter_at_location(x, y)
+    if dad.lists_handler.is_list_todo_beginning(text_iter):
+        text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.X_CURSOR))
+        text_view.set_tooltip_text(None)
+        return
+    tags = text_iter.get_tags()
+    if not tags: tags = []
+    for tag in tags:
+        tag_name = tag.get_property("name")
+        if tag_name and tag_name[0:4] == cons.TAG_LINK:
+            hovering_link = True
+            tooltip = dad.sourceview_hovering_link_get_tooltip(tag_name[5:])
+            break
+    else:
+        iter_anchor = text_iter.copy()
+        for i in [0, 1]:
+            if i == 1: iter_anchor.backward_char()
+            anchor = iter_anchor.get_child_anchor()
+            if anchor and "pixbuf" in dir(anchor):
+                pixbuf_attrs = dir(anchor.pixbuf)
+                if "link" in pixbuf_attrs and anchor.pixbuf.link:
+                    hovering_link = True
+                    tooltip = dad.sourceview_hovering_link_get_tooltip(anchor.pixbuf.link)
+                    break
+    if hovering_link != dad.hovering_over_link:
+        dad.hovering_over_link = hovering_link
+        #print "link", dad.hovering_over_link
+    if dad.hovering_over_link:
+        text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
+        text_view.set_tooltip_text(tooltip)
+    else:
+        text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
+        text_view.set_tooltip_text(None)
+
+def rich_text_node_modify_codeboxes_font(start_iter, code_font):
+    """Modify Font to CodeBoxes"""
+    curr_iter = start_iter.copy()
+    while 1:
+        anchor = curr_iter.get_child_anchor()
+        if anchor and "sourcebuffer" in dir(anchor):
+            anchor.sourceview.modify_font(pango.FontDescription(code_font))
+        if not curr_iter.forward_char(): break
+
 def text_file_rm_emptylines(filepath):
     """Remove empty lines in a text file"""
     overwrite_needed = False
@@ -39,6 +239,22 @@ def text_file_rm_emptylines(filepath):
         fd.writelines(file_lines)
         fd.close()
 
+def get_proper_platform_filepath(filepath_in, is_file):
+    """From Slash to Backslash when needed"""
+    filepath_out = ""
+    if cons.IS_WIN_OS:
+        if cons.CHAR_SLASH in filepath_in:
+            filepath_out = filepath_in.replace(cons.CHAR_SLASH, cons.CHAR_BSLASH)
+    else:
+        if cons.CHAR_BSLASH in filepath_in:
+            filepath_out = filepath_in.replace(cons.CHAR_BSLASH, cons.CHAR_SLASH)
+    if filepath_out:
+        if is_file:
+            if os.path.isfile(filepath_out): return filepath_out
+        else:
+            if os.path.isdir(filepath_out): return filepath_out
+    return filepath_in
+
 def clean_from_chars_not_for_filename(filename_in):
     """Clean a string from chars not good for filename"""
     filename_out = filename_in.replace(cons.CHAR_SLASH, cons.CHAR_MINUS).replace(cons.CHAR_BSLASH, cons.CHAR_MINUS)
@@ -48,16 +264,20 @@ def clean_from_chars_not_for_filename(filename_in):
     filename_out = filename_out.replace(cons.CHAR_NEWLINE, "").replace(cons.CHAR_CR, "").strip()
     return filename_out.replace(cons.CHAR_SPACE, cons.CHAR_USCORE)
 
-def get_node_hierarchical_name(dad, tree_iter, separator="--"):
+def get_node_hierarchical_name(dad, tree_iter, separator="--", for_filename=True, root_to_leaf=True):
     """Get the Node Hierarchical Name"""
     hierarchical_name = dad.treestore[tree_iter][1].strip()
     father_iter = dad.treestore.iter_parent(tree_iter)
     while father_iter:
-        hierarchical_name = dad.treestore[father_iter][1].strip() + separator + hierarchical_name
+        if root_to_leaf:
+            hierarchical_name = dad.treestore[father_iter][1].strip() + separator + hierarchical_name
+        else:
+            hierarchical_name = hierarchical_name + separator + dad.treestore[father_iter][1].strip()
         father_iter = dad.treestore.iter_parent(father_iter)
-    hierarchical_name = clean_from_chars_not_for_filename(hierarchical_name)
-    if len(hierarchical_name) > cons.MAX_FILE_NAME_LEN:
-        hierarchical_name = hierarchical_name[-cons.MAX_FILE_NAME_LEN:]
+    if for_filename:
+        hierarchical_name = clean_from_chars_not_for_filename(hierarchical_name)
+        if len(hierarchical_name) > cons.MAX_FILE_NAME_LEN:
+            hierarchical_name = hierarchical_name[-cons.MAX_FILE_NAME_LEN:]
     return hierarchical_name
 
 def strip_trailing_spaces(text_buffer):
@@ -291,7 +511,7 @@ MA 02110-1301, USA.
     dialog.set_documenters(["Robert Boudreau <RobtTheB@gmail.com>"])
     dialog.set_artists(["OCAL <http://www.openclipart.org/>", "Zeltak <zeltak@gmail.com>", "Angelo Penone <angelo.penone@gmail.com>"])
     dialog.set_translator_credits(_("Armenian")+" (hy) Seda Stamboltsyan <sedastam@yandex.com>"+cons.CHAR_NEWLINE+
-_("Chinese Simplified")+" (zh_CN) Channing Wong <mamimoluo@gmail.com>"+cons.CHAR_NEWLINE+
+_("Chinese Simplified")+" (zh_CN) Channing Wong <channing.wong@qq.com>"+cons.CHAR_NEWLINE+
 _("Czech")+" (cs) Pavel Fric <fripohled@blogspot.com>"+cons.CHAR_NEWLINE+
 _("Dutch")+" (nl) Patrick Vijgeboom <pj.vijgeboom@gmail.com>"+cons.CHAR_NEWLINE+
 _("French")+" (fr) Benoît D'Angelo <benoit.dangelo@gmx.fr>"+cons.CHAR_NEWLINE+
@@ -307,10 +527,13 @@ _("Ukrainian")+" (uk) Andriy Kovtun <kovtunos@yandex.ru>")
     dialog.set_transient_for(dad.window)
     dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
     dialog.set_modal(True)
+    def f_url_hook(dialog, link, user_data):
+        webbrowser.open(link)
+    gtk.about_dialog_set_url_hook(f_url_hook, None)
     dialog.run()
     dialog.hide()
 
-def dialog_choose_element_in_list(father_win, title, elements_list, column_title):
+def dialog_choose_element_in_list(father_win, title, elements_list, column_title, icon_n_label_list=None):
     """Choose Between Elements in List"""
     class ListParms:
         def __init__(self):
@@ -325,14 +548,29 @@ def dialog_choose_element_in_list(father_win, title, elements_list, column_title
     dialog.set_default_size(400, 300)
     scrolledwindow = gtk.ScrolledWindow()
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-    elements_liststore = gtk.ListStore(str)
+    if not icon_n_label_list: elements_liststore = gtk.ListStore(str)
+    else: elements_liststore = gtk.ListStore(str,str,str)
     elements_treeview = gtk.TreeView(elements_liststore)
     elements_renderer_text = gtk.CellRendererText()
-    elements_column = gtk.TreeViewColumn(column_title, elements_renderer_text, text=0)
+    if not icon_n_label_list:
+        elements_column = gtk.TreeViewColumn(column_title, elements_renderer_text, text=0)
+    else:
+        elements_treeview.set_headers_visible(False)
+        renderer_pixbuf = gtk.CellRendererPixbuf()
+        renderer_text = gtk.CellRendererText()
+        elements_column = gtk.TreeViewColumn()
+        elements_column.pack_start(renderer_pixbuf, False)
+        elements_column.pack_start(renderer_text, True)
+        elements_column.set_attributes(renderer_pixbuf, stock_id=1)
+        elements_column.set_attributes(renderer_text, text=2)
     elements_treeview.append_column(elements_column)
     elements_treeviewselection = elements_treeview.get_selection()
-    for element_name in elements_list:
-        elements_liststore.append(element_name)
+    if not icon_n_label_list:
+        for element_name in elements_list:
+            elements_liststore.append(element_name)
+    else:
+        for element in icon_n_label_list:
+            elements_liststore.append([element[0], element[1], element[2]])
     scrolledwindow.add(elements_treeview)
     list_parms.sel_iter = elements_liststore.get_iter_first()
     if list_parms.sel_iter:
@@ -538,6 +776,49 @@ def dialog_node_delete(father_win, warning_label):
     dialog.hide()
     return response
 
+def dialog_exit_del_temp_files(dad):
+    """Temporary Files will be deleted, close applications using them"""
+    dialog = gtk.Dialog(title=_("Warning"),
+        parent=dad.window,
+        flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
+        buttons=(_("Cancel"), 2,
+                 _("Yes"), 1) )
+    dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+    dialog.set_default_size(350, 150)
+    try:
+        button = dialog.get_widget_for_response(2)
+        button.set_image(gtk.image_new_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON))
+        button = dialog.get_widget_for_response(1)
+        button.set_image(gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_BUTTON))
+        button.grab_focus()
+    except: pass
+    image = gtk.Image()
+    image.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
+    label = gtk.Label("<b>"+_("Temporary Files were Created and Opened with External Applications.")+"</b>"+2*cons.CHAR_NEWLINE+"<b>"+_("Quit the External Applications Before Quit CherryTree.")+"</b>"+2*cons.CHAR_NEWLINE+"<b>"+_("Did you Quit the External Applications?")+"</b>")
+    label.set_use_markup(True)
+    hbox = gtk.HBox()
+    hbox.pack_start(image)
+    hbox.pack_start(label)
+    hbox.set_spacing(5)
+    content_area = dialog.get_content_area()
+    content_area.pack_start(hbox)
+    def on_key_press_exitdialog(widget, event):
+        keyname = gtk.gdk.keyval_name(event.keyval)
+        if keyname == cons.STR_RETURN:
+            try: dialog.get_widget_for_response(1).clicked()
+            except: print cons.STR_PYGTK_222_REQUIRED
+            return True
+        elif keyname == "Escape":
+            try: dialog.get_widget_for_response(2).clicked()
+            except: print cons.STR_PYGTK_222_REQUIRED
+            return True
+        return False
+    dialog.connect('key_press_event', on_key_press_exitdialog)
+    content_area.show_all()
+    response = dialog.run()
+    dialog.hide()
+    return False if response != 1 else True
+
 def dialog_exit_save(father_win):
     """Save before Exit Dialog"""
     dialog = gtk.Dialog(title=_("Warning"),
@@ -559,8 +840,7 @@ def dialog_exit_save(father_win):
     except: pass
     image = gtk.Image()
     image.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
-    label = gtk.Label(_("""<b>The current document was updated</b>,
-do you want to save the changes?"""))
+    label = gtk.Label("<b>"+_("The Current Document was Updated.")+"</b>"+2*cons.CHAR_NEWLINE+"<b>"+_("Do you want to Save the Changes?")+"</b>")
     label.set_use_markup(True)
     hbox = gtk.HBox()
     hbox.pack_start(image)
@@ -665,6 +945,8 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     scrolledwindow = gtk.ScrolledWindow()
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
     scrolledwindow.add(treeview_2)
+    config.get_tree_expanded_collapsed_string(dad)
+    config.set_tree_expanded_collapsed_string(dad, treeview=treeview_2)
     if links_parms.sel_iter:
         sel_path = dad.treestore.get_path(links_parms.sel_iter)
         treeview_2.expand_to_path(sel_path)
@@ -729,11 +1011,17 @@ def dialog_link_handle(dad, title, sel_tree_iter):
         filepath = dialog_file_select(curr_folder=dad.pick_dir, parent=dialog)
         if not filepath: return
         dad.pick_dir = os.path.dirname(filepath)
+        if dad.links_relative:
+            try: filepath = os.path.relpath(filepath, os.getcwd())
+            except: print "cannot set relative path for different drives"
         entry_file.set_text(filepath)
     def on_button_browse_for_folder_to_link_to_clicked(self, *args):
         filepath = dialog_folder_select(curr_folder=dad.pick_dir, parent=dialog)
         if not filepath: return
         dad.pick_dir = filepath
+        if dad.links_relative:
+            try: filepath = os.path.relpath(filepath, os.getcwd())
+            except: print "cannot set relative path for different drives"
         entry_folder.set_text(filepath)
     def on_browse_anchors_button_clicked(*args):
         if not links_parms.sel_iter:
@@ -755,8 +1043,22 @@ def dialog_link_handle(dad, title, sel_tree_iter):
         ret_anchor_name = dialog_choose_element_in_list(dialog, _("Choose Existing Anchor"), anchors_list, _("Anchor Name"))
         if ret_anchor_name: entry_anchor.set_text(ret_anchor_name)
     def on_treeview_event_after(treeview, event):
-        if event.type not in [gtk.gdk.BUTTON_PRESS, gtk.gdk.KEY_PRESS]: return
+        if event.type not in [gtk.gdk.BUTTON_PRESS, gtk.gdk._2BUTTON_PRESS, gtk.gdk.KEY_PRESS]: return
         model, links_parms.sel_iter = treeviewselection_2.get_selected()
+        if event.type == gtk.gdk.BUTTON_PRESS:
+            if event.button == 2:
+                path_at_click = treeview.get_path_at_pos(int(event.x), int(event.y))
+                if path_at_click:
+                    if treeview.row_expanded(path_at_click[0]):
+                        treeview.collapse_row(path_at_click[0])
+                    else: treeview.expand_row(path_at_click[0], False)
+        elif event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
+            if links_parms.sel_iter:
+                treestore = treeview.get_model()
+                if treeview.row_expanded(treestore.get_path(links_parms.sel_iter)):
+                    treeview.collapse_row(treestore.get_path(links_parms.sel_iter))
+                else:
+                    treeview.expand_row(treestore.get_path(links_parms.sel_iter), open_all=False)
     def on_key_press_links_handle_dialog(widget, event):
         if gtk.gdk.keyval_name(event.keyval) == cons.STR_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
@@ -785,14 +1087,14 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     dad.links_entries['node'] = links_parms.sel_iter
     return True
 
-def dialog_choose_node(father_win, title, treestore, sel_tree_iter):
+def dialog_choose_node(dad, title, treestore, sel_tree_iter):
     """Dialog to Select a Node"""
     class NodeParms:
         def __init__(self):
             self.sel_iter = sel_tree_iter
     node_parms = NodeParms()
     dialog = gtk.Dialog(title=title,
-        parent=father_win,
+        parent=dad.window,
         flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
         buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
         gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
@@ -812,6 +1114,8 @@ def dialog_choose_node(father_win, title, treestore, sel_tree_iter):
     scrolledwindow = gtk.ScrolledWindow()
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
     scrolledwindow.add(treeview_2)
+    config.get_tree_expanded_collapsed_string(dad)
+    config.set_tree_expanded_collapsed_string(dad, treeview=treeview_2)
     if node_parms.sel_iter:
         sel_path = treestore.get_path(node_parms.sel_iter)
         treeview_2.expand_to_path(sel_path)
@@ -825,16 +1129,24 @@ def dialog_choose_node(father_win, title, treestore, sel_tree_iter):
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
         return False
-    def on_mouse_button_clicked_treeview_2(widget, event):
-        if event.button != 1: return
-        if event.type == gtk.gdk._2BUTTON_PRESS:
-            try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
-            except: print cons.STR_PYGTK_222_REQUIRED
     def on_treeview_event_after(treeview, event):
-        if event.type not in [gtk.gdk.BUTTON_PRESS, gtk.gdk.KEY_PRESS]: return
+        if event.type not in [gtk.gdk.BUTTON_PRESS, gtk.gdk._2BUTTON_PRESS, gtk.gdk.KEY_PRESS]: return
         model, node_parms.sel_iter = treeviewselection_2.get_selected()
+        if event.type == gtk.gdk.BUTTON_PRESS:
+            if event.button == 2:
+                path_at_click = treeview.get_path_at_pos(int(event.x), int(event.y))
+                if path_at_click:
+                    if treeview.row_expanded(path_at_click[0]):
+                        treeview.collapse_row(path_at_click[0])
+                    else: treeview.expand_row(path_at_click[0], False)
+        elif event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
+            if node_parms.sel_iter:
+                treestore = treeview.get_model()
+                if treeview.row_expanded(treestore.get_path(node_parms.sel_iter)):
+                    treeview.collapse_row(treestore.get_path(node_parms.sel_iter))
+                else:
+                    treeview.expand_row(treestore.get_path(node_parms.sel_iter), open_all=False)
     dialog.connect("key_press_event", on_key_press_choose_node_dialog)
-    treeview_2.connect('button-press-event', on_mouse_button_clicked_treeview_2)
     treeview_2.connect('event-after', on_treeview_event_after)
     content_area.show_all()
     response = dialog.run()
@@ -977,14 +1289,15 @@ def set_menu_items_recent_documents(inst):
         recent_menuitem.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
         recent_menuitem.set_tooltip_text(_("Open a Recent CherryTree Document"))
         recent_menuitem.set_submenu(inst.recent_menu_1)
-        inst.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 9)
+        inst.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 3)
         # toolbar
-        menu_toolbutton = gtk.MenuToolButton("gtk-open")
-        menu_toolbutton.set_tooltip_text(_("Open a CherryTree Document"))
-        menu_toolbutton.set_arrow_tooltip_text(_("Open a Recent CherryTree Document"))
-        menu_toolbutton.set_menu(inst.recent_menu_2)
-        menu_toolbutton.connect("clicked", inst.file_open)
-        inst.ui.get_widget("/ToolBar").insert(menu_toolbutton, 8)
+        if inst.toolbar_open_n_recent >= 0:
+            menu_toolbutton = gtk.MenuToolButton("gtk-open")
+            menu_toolbutton.set_tooltip_text(_("Open a CherryTree Document"))
+            menu_toolbutton.set_arrow_tooltip_text(_("Open a Recent CherryTree Document"))
+            menu_toolbutton.set_menu(inst.recent_menu_2)
+            menu_toolbutton.connect("clicked", inst.file_open)
+            inst.ui.get_widget("/ToolBar").insert(menu_toolbutton, inst.toolbar_open_n_recent)
 
 def add_recent_document(inst, filepath):
     """Add a Recent Document if Needed"""
@@ -996,8 +1309,9 @@ def add_recent_document(inst, filepath):
 
 def insert_special_char(menu_item, special_char, dad):
     """A Special character insert was Requested"""
-    if not dad.is_there_selected_node_or_error(): return
-    dad.curr_buffer.insert_at_cursor(special_char)
+    text_view, text_buffer, from_codebox = dad.get_text_view_n_buffer_codebox_proof()
+    if not text_buffer: return
+    text_buffer.insert_at_cursor(special_char)
 
 def open_recent_document(menu_item, filepath, dad):
     """A Recent Document was Requested"""
